/**************************************************************************************

        PROTUX - THE FREE PROFESSIONAL AUDIO TOOLS FOR LINUX
        AUTHOR : See AUTHORS file for details

        This software is distributed under the terms of the GNU General Public License
        as specified in the COPYING file.

***************************************************************************************/
#include "GlobalPropertiesWindow.hh"
#include "GlobalProperties.hh"
#include <mustux.h>

#include <qcombobox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qspinbox.h>
#include <qtabwidget.h>
#include <qwidget.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>

/*
*  Constructs a GlobalProperties which is a child of 'parent', with the
*  name 'name' and widget flags set to 'f'
*
*  The dialog will by default be modeless, unless you set 'modal' to
*  TRUE to construct a modal dialog.
*/
GlobalPropertiesWindow::GlobalPropertiesWindow( Interface* pParentInterface,  const char* name )
	: QWidget( (QWidget*) 0 , name )
        {
        PENTERCONS;
        parentInterface = pParentInterface;
        if ( !name )
                setName( "GlobalProperties" );
        resize( 592, 480 );
        setCaption( tr( "Global Properties Window" ) );


        QPushButton* buttonApply = new QPushButton( this );
        buttonApply->setGeometry( QRect( 110, 440, 102, 26 ) );
        buttonApply->setText( tr( "APPLY" ) );
        QPushButton* buttonOk = new QPushButton( this );
        buttonOk->setGeometry( QRect( 240, 440, 102, 26 ) );
        buttonOk->setText( tr( "OK (Save)" ) );

        connect( buttonApply, SIGNAL(clicked()), this, SLOT(apply()) );
        connect( buttonOk, SIGNAL(clicked()), this, SLOT(ok()) );

        QTabWidget* TabWidget1 = new QTabWidget( this);
        TabWidget1->setGeometry( QRect( 0, 0, 600, 430 ) );
	//TabWidget1->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)4, (QSizePolicy::SizeType)4, TabWidget1->sizePolicy().hasHeightForWidth() ) );
	QToolTip::add(  TabWidget1, tr( "" ) );
        // DSP

        QWidget* tab1 = new QWidget( TabWidget1 );

        QWidget* privateLayoutWidget1 = new QWidget( tab1  );
        privateLayoutWidget1->setGeometry( QRect( 60, 60, 370, 24 ) );
        QHBoxLayout* Layout1 = new QHBoxLayout( privateLayoutWidget1 );
        Layout1->setSpacing( 6 );
        Layout1->setMargin( 0 );
        QLabel* TextLabel1 = new QLabel( privateLayoutWidget1 );
        //TextLabel1->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)1, TextLabel1->sizePolicy().hasHeightForWidth() ) );
        TextLabel1->setText( tr( "Dsp Device" ) );
        Layout1->addWidget( TextLabel1 );
        wStandardDevice = new QLineEdit( privateLayoutWidget1 );
        wStandardDevice->setText( tr( GlobalProperties::get("DSP_DEVICE") ) );
        Layout1->addWidget( wStandardDevice );

        QWidget* privateLayoutWidget2 = new QWidget( tab1 );
        privateLayoutWidget2->setGeometry( QRect( 60, 170, 330, 25 ) );
        QHBoxLayout* Layout2 = new QHBoxLayout( privateLayoutWidget2 );
        Layout2->setSpacing( 6 );
        Layout2->setMargin( 0 );
        QLabel* TextLabel2 = new QLabel( privateLayoutWidget2 );
        //TextLabel2->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)1, TextLabel2->sizePolicy().hasHeightForWidth() ) );
        TextLabel2->setText( tr( "Slice Width Factor" ) );
        TextLabel2->setGeometry( QRect( 150, 200, 170, 22 ) );
        Layout2->addWidget( TextLabel2 );
        wSliceWidthFactor = new QSpinBox( privateLayoutWidget2 );
        //wSliceWidthFactor->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)0, wSliceWidthFactor->sizePolicy().hasHeightForWidth() ) );
        QToolTip::add(  wSliceWidthFactor, tr( "How many dsp fragments of audio will be mixed per slice" ) );
        wSliceWidthFactor->setValue(GlobalProperties::get_int("SLICE_WIDTH_FACTOR"));
        Layout2->addWidget( wSliceWidthFactor );


        QWidget* privateLayoutWidget3 = new QWidget( tab1 );
        privateLayoutWidget3->setGeometry( QRect( 60, 290, 350, 25 ) );
        QHBoxLayout* Layout3 = new QHBoxLayout( privateLayoutWidget3 );
        Layout3->setSpacing( 6 );
        Layout3->setMargin( 0 );
        QLabel* TextLabel3 = new QLabel( privateLayoutWidget3 );
        //TextLabel3->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)1, TextLabel3->sizePolicy().hasHeightForWidth() ) );
        TextLabel3->setText( tr( "Sample Rate" ) );
        Layout3->addWidget( TextLabel3 );
        wDefaultSampleRate = new QComboBox( FALSE, privateLayoutWidget3 );
        wDefaultSampleRate->insertItem( tr( "22050" ) );
        wDefaultSampleRate->insertItem( tr( "44100" ) );
        wDefaultSampleRate->insertItem( tr( "48000" ) );
        wDefaultSampleRate->insertItem( tr( "96000" ) );
        wDefaultSampleRate->setCurrentItem( 2);// FOR LATERGlobalProperties::get_int("DEFAULT_SAMPLE_RATE");
        Layout3->addWidget( wDefaultSampleRate );


        QWidget* privateLayoutWidget4 = new QWidget( tab1 );
        privateLayoutWidget4->setGeometry( QRect( 60, 320, 350, 25 ) );
        QHBoxLayout* Layout4 = new QHBoxLayout( privateLayoutWidget4 );
        Layout4->setSpacing( 6 );
        Layout4->setMargin( 0 );
        QLabel* TextLabel4 = new QLabel( privateLayoutWidget4 );
        //TextLabel4->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)1, TextLabel4->sizePolicy().hasHeightForWidth() ) );
        TextLabel4->setText( tr( "Bit Depth" ) );
        Layout4->addWidget( TextLabel4 );
        wDefaultBitDepth = new QComboBox( FALSE, privateLayoutWidget4  );
        wDefaultBitDepth->insertItem( tr( "16" ) );
        wDefaultBitDepth->insertItem( tr( "20" ) );
        wDefaultBitDepth->insertItem( tr( "24" ) );
        wDefaultBitDepth->insertItem( tr( "32" ) );
        wDefaultBitDepth->setCurrentItem( 2); // FOR LATERGlobalProperties::get_int("DEFAULT_BIT_DEPTH");
        Layout4->addWidget( wDefaultBitDepth );


        TabWidget1->insertTab( tab1, tr( "DSP" ) );



        // JMB

        QWidget* tab2 = new QWidget( TabWidget1 );

        QLabel* TextLabel5 = new QLabel( tab2 );
        TextLabel5->setGeometry( QRect( 30, 30, 200, 20 ) );
        TextLabel5->setText( tr( "Lcd Clear Time" ) );

        QLabel* TextLabel51 = new QLabel( tab2 );
        TextLabel51->setGeometry( QRect( 30, 80, 200, 20 ) );
        TextLabel51->setText( tr( "Double Press Sensitiveness" ) );

        QLabel* TextLabel52 = new QLabel( tab2 );
        TextLabel52->setGeometry( QRect( 28, 135, 200, 20 ) );
        TextLabel52->setText( tr( "Hold Sensitiveness" ) );

        QPushButton* PushButton5 = new QPushButton( tab2 );
        PushButton5->setGeometry( QRect( 350, 350, 180, 30 ) );
        PushButton5->setText( tr( "View JMB Map" ) );

        wJmbClearTime = new QSpinBox( tab2 );
        wJmbClearTime->setGeometry( QRect( 250, 30, 97, 23 ) );
        wJmbClearTime->setMaxValue(5000);
        wJmbClearTime->setMinValue(500);
        wJmbClearTime->setValue(GlobalProperties::get_int("JMB_CLEAR_TIME"));

        wJmbHoldSensitivenessFactor = new QSpinBox( tab2 );
        wJmbHoldSensitivenessFactor->setGeometry( QRect( 250, 85, 97, 23 ) );
        wJmbHoldSensitivenessFactor->setMinValue(1);
        wJmbHoldSensitivenessFactor->setMaxValue(50);
        wJmbHoldSensitivenessFactor->setValue(GlobalProperties::get_int("JMB_HOLD_SENSITIVENESS_FACTOR"));

        wJmbDoublePressInterval = new QSpinBox( tab2 );
        wJmbDoublePressInterval->setGeometry( QRect( 250, 135, 97, 23 ) );
        wJmbDoublePressInterval->setMinValue(50);
        wJmbDoublePressInterval->setMaxValue(1000);
        wJmbDoublePressInterval->setValue(GlobalProperties::get_int("JMB_DOUBLE_PRESS_INTERVAL"));



        TabWidget1->insertTab( tab2, tr( "JMB" ) );


        // EXTERNAL PROGRAMS
        QWidget* tab3 = new QWidget( TabWidget1 );

        QWidget* privateLayoutWidget6 = new QWidget( tab3 );
        privateLayoutWidget6->setGeometry( QRect( 60, 60, 420, 32 ) );
        QHBoxLayout* Layout6 = new QHBoxLayout( privateLayoutWidget6 );
        Layout6->setSpacing( 6 );
        Layout6->setMargin( 0 );
        QLabel* TextLabel6 = new QLabel( privateLayoutWidget6 );
        TextLabel6->setText( tr( "Mp3 decode command" ) );
        Layout6->addWidget( TextLabel6 );
        wMp3DecodeCommand= new QLineEdit( privateLayoutWidget6 );
        wMp3DecodeCommand->setText( tr( GlobalProperties::get("MP3_DECODE_COMMAND")) );
        Layout6->addWidget( wMp3DecodeCommand);


        QWidget* privateLayoutWidget7 = new QWidget( tab3 );
        privateLayoutWidget7->setGeometry( QRect( 60, 120, 420, 32 ) );
        QHBoxLayout* Layout7 = new QHBoxLayout( privateLayoutWidget7 );
        Layout7->setSpacing( 6 );
        Layout7->setMargin( 0 );
        QLabel* TextLabel7 = new QLabel( privateLayoutWidget7 );
        TextLabel7->setText( tr( "Mp3 encode command" ) );
        Layout7->addWidget( TextLabel7 );
        wMp3EncodeCommand = new QLineEdit( privateLayoutWidget7 );
        wMp3EncodeCommand ->setText( tr( GlobalProperties::get("MP3_ENCODE_COMMAND") ) );
        Layout7->addWidget( wMp3EncodeCommand);


        QWidget* privateLayoutWidget8 = new QWidget( tab3 );
        privateLayoutWidget8->setGeometry( QRect( 58, 245, 420, 32 ) );
        QHBoxLayout* Layout8 = new QHBoxLayout( privateLayoutWidget8 );
        Layout8->setSpacing( 6 );
        Layout8->setMargin( 0 );
        QLabel* TextLabel8 = new QLabel( privateLayoutWidget8 );
        TextLabel8->setText( tr( "Ogg encode command" ) );
        Layout8->addWidget( TextLabel8 );
        wOggEncodeCommand= new QLineEdit( privateLayoutWidget8 );
        wOggEncodeCommand->setText( tr( GlobalProperties::get("OGG_ENCODE_COMMAND")) );
        Layout8->addWidget( wOggEncodeCommand);


        QWidget* privateLayoutWidget9 = new QWidget( tab3 );
        privateLayoutWidget9->setGeometry( QRect( 60, 180, 420, 32 ) );
        QHBoxLayout* Layout9 = new QHBoxLayout( privateLayoutWidget9 );
        Layout9->setSpacing( 6 );
        Layout9->setMargin( 0 );
        QLabel* TextLabel9 = new QLabel( privateLayoutWidget9 );
        TextLabel9->setText( tr( "Ogg decode command" ) );
        Layout9->addWidget( TextLabel9 );
        wOggDecodeCommand= new QLineEdit( privateLayoutWidget9 );
        wOggDecodeCommand->setText( tr( GlobalProperties::get("OGG_DECODE_COMMAND") ) );
        Layout9->addWidget( wOggDecodeCommand);
        QLabel* TextLabel91 = new QLabel( tab3 );
        TextLabel91->setGeometry( QRect( 48, 315, 470, 40 ) );
        TextLabel91->setText( tr( "Note: \"%if\"  == input file  / \"%of\" == output file" ) );

        TabWidget1->insertTab( tab3, tr( "External Programs" ) );



        //  MTA


        QWidget* tab4 = new QWidget( TabWidget1 );

        QLabel* TextLabel10 = new QLabel( tab4 );
        TextLabel10->setGeometry( QRect( 41, 31, 190, 30 ) );
        TextLabel10->setText( tr( "Default Number of Tracks" ) );
        wDefaultNumTracks = new QSpinBox( tab4 );
        wDefaultNumTracks->setGeometry( QRect( 250, 30, 97, 23 ) );
        wDefaultNumTracks->setMinValue(1);
        wDefaultNumTracks->setMaxValue(16);
        wDefaultNumTracks->setValue(GlobalProperties::get_int("DEFAULT_NUM_TRACKS"));

        QLabel* TextLabel101 = new QLabel( tab4 );
        TextLabel101->setGeometry( QRect( 38, 85, 190, 30 ) );
        TextLabel101->setText( tr( "Default H Zoom Level" ) );
        wDefaultHZoomLevel = new QComboBox( FALSE, tab4 );
        wDefaultHZoomLevel->insertItem( tr("0") );// "1:1" ) );
        wDefaultHZoomLevel->insertItem( tr("1") );// "1:2" ) );
        wDefaultHZoomLevel->insertItem( tr("2") );// "1:4" ) );
        wDefaultHZoomLevel->insertItem( tr("3") );// "1:8" ) );
        wDefaultHZoomLevel->insertItem( tr("4") );// "1:12" ) );
        wDefaultHZoomLevel->insertItem( tr("5") );// "1:16" ) );
        wDefaultHZoomLevel->insertItem( tr("6") );// "1:24" ) );
        wDefaultHZoomLevel->insertItem( tr("7") );// "1:32" ) );
        wDefaultHZoomLevel->insertItem( tr("8") );// "1:48" ) );
        wDefaultHZoomLevel->insertItem( tr("9") );// "1:64" ) );
        wDefaultHZoomLevel->insertItem( tr("10") );// "1:96" ) );
        wDefaultHZoomLevel->insertItem( tr("11") );// "1:128" ) );
        wDefaultHZoomLevel->insertItem( tr("12") );// "1:192" ) );
        wDefaultHZoomLevel->insertItem( tr("13") );// "1:256" ) );
        wDefaultHZoomLevel->insertItem( tr("14") );// "1:384" ) );
        wDefaultHZoomLevel->insertItem( tr("15") );// "1:512" ) );
        wDefaultHZoomLevel->insertItem( tr("16") );// "1:1024" ) );
        wDefaultHZoomLevel->insertItem( tr("17") );// "1:2048" ) );
        wDefaultHZoomLevel->insertItem( tr("18") );// "1:4096" ) );
        wDefaultHZoomLevel->insertItem( tr("19") );// "1:8192" ) );
        wDefaultHZoomLevel->insertItem( tr("20") );// "1:16384" ) );
        wDefaultHZoomLevel->insertItem( tr("21") );// "1:32768" ) );
        wDefaultHZoomLevel->insertItem( tr("22") );// "1:65536" ) );

        wDefaultHZoomLevel->setCurrentItem( GlobalProperties::get_int("HZOOM_LEVEL"));
        wDefaultHZoomLevel->setGeometry( QRect( 250, 90, 88, 22 ) );

        TabWidget1->insertTab( tab4, tr( "MTA" ) );

        PEXITCONS;
        }

/*
*  Destroys the object and frees any allocated resources
*/
GlobalPropertiesWindow::~GlobalPropertiesWindow()
        {
        PENTERDES;
        // no need to delete child widgets, Qt does it all for us
        PEXITDES;
        }

void GlobalPropertiesWindow::apply()
        {
        PENTER;
        PMESG("Applying properties...");
        GlobalProperties::set("DSP_DEVICE", (char*) wStandardDevice->text().latin1());
        GlobalProperties::set("MP3_DECODE_COMMAND",(char*) wMp3DecodeCommand->text().latin1());
        GlobalProperties::set("OGG_DECODE_COMMAND",(char*) wOggDecodeCommand->text().latin1());
        GlobalProperties::set("MP3_ENCODE_COMMAND",(char*) wMp3EncodeCommand->text().latin1());
        GlobalProperties::set("OGG_ENCODE_COMMAND",(char*) wOggEncodeCommand->text().latin1());
        GlobalProperties::set("SLICE_WIDTH_FACTOR",(char*) wSliceWidthFactor->text().latin1());
        GlobalProperties::set("DEFAULT_SAMPLE_RATE",(char*) wDefaultSampleRate->currentText().latin1());
        GlobalProperties::set("DEFAULT_BIT_DEPTH",(char*) wDefaultBitDepth->currentText().latin1());
        GlobalProperties::set("JMB_CLEAR_TIME",(char*) wJmbClearTime->text().latin1());
        GlobalProperties::set("JMB_HOLD_SENSITIVENESS_FACTOR",(char*) wJmbHoldSensitivenessFactor->text().latin1());
        GlobalProperties::set("JMB_DOUBLE_PRESS_INTERVAL",(char*) wJmbDoublePressInterval->text().latin1());
        GlobalProperties::set("DEFAULT_NUM_TRACKS",(char*) wDefaultNumTracks->text().latin1());
        GlobalProperties::set("HZOOM_LEVEL",(char*) wDefaultHZoomLevel->currentText().latin1());

        parentInterface->update_properties();
        PEXIT;
        }


void GlobalPropertiesWindow::ok()
        {
        PENTER;
        apply();
        GlobalProperties::save();
        this->hide();
        PEXIT;
        }

//eof

