/**************************************************************************************

        PROTUX - THE FREE PROFESSIONAL AUDIO TOOLS FOR LINUX
        AUTHOR : See AUTHORS file for details

        This software is distributed under the terms of the GNU General Public License
        as specified in the COPYING file.

***************************************************************************************/
#include <stdio.h>
#include <qprogressdialog.h>
#include <mustux.h>
#include "Mixer.hh"
#include "Track.hh"
#include "PrafFile.hh"


// TEM MUITA DUPLICACAO DE VARIAVEIS ENTRE MIXER E SLICE. MAX CHANNELS, SLICEWIDTHINBYUTES, ETC...
// ALL THIS SLICE STUFF MUST BE REVIEWS. DIFERENT BUSES HAVE
// DIFERENT FRAGMENTSIZE SO , THE SLICING CONCEPT MIGHT BE DEPRECATED !!!



Mixer::Mixer(Song* pSong)
        {
        PENTERCONS;
        song = pSong;
        lcd = song->get_default_lcd();
        firstSlice = (Slice*) 0 ;
        isPlaying=false;
        playStartBlock=0;
        thPlayLoop = new ThPlayLoop(this);
        maxChannels = 2; // IMPROVEME
        busMonitor = song->get_bus_monitor();

        // Calculate the max mixBufferSize in order to allocate the mixBuffers
        fragSize = MustuxAudioDeviceMapper::get_fragment_size(MustuxAudioDeviceMapper::DEFAULT_BUS);
        sliceWidthFactor = GlobalProperties::get_int("SLICE_WIDTH_FACTOR");
        sliceWidth = fragSize * sliceWidthFactor; // must be a multiple of fragSize
        sliceWidthInBytes = ((sliceWidth) * maxChannels) * sizeof(short);

        for (int bus = 0; bus<MustuxAudioDeviceMapper::get_number_of_buses(); bus++)
                {
                mixBuffer[bus] = new char[sliceWidthInBytes];
                mixBufferSize[bus] = sliceWidthInBytes; // does it really need to be an array (all is the same size!!!)
                }

        renderBuffer = new char[sliceWidthInBytes];
        renderBufferSize = sliceWidthInBytes;
        PEXITCONS;
        }



Mixer::~Mixer()
        {
        PENTERDES;
        Slice* s = firstSlice;
        Slice* s2= firstSlice;
        while (s)
                {
                s2=s->next;
                delete s;
                s=s2;
                }
        for (int bus = 0; bus<MustuxAudioDeviceMapper::get_number_of_buses(); bus++)
                {
                if (mixBuffer[bus])
                        delete mixBuffer[bus];
                }
        PEXITDES;
        }





int Mixer::play(long long pPlayStartBlock)
        {
        PENTER;
        long long lastMtaBlock = song->get_last_song_block();
        if (lastMtaBlock==0)
                {
                PMESG("MTA SEEMS TO HAVE NO CONTENT !");
                PEXIT;
                return -1;
                }

        PMESG("Starting PLAYBACK at pPlayStartBlock=%d",pPlayStartBlock);
        playStartBlock = pPlayStartBlock;

        //  IMPROVE ME !!
        int rate=44100;
        int channels=2;
        int bitDepth=16;
        // -------------

        int nBuses = MustuxAudioDeviceMapper::get_number_of_buses();
        busStatus = new bool[nBuses];
        for (int bus=0; bus<nBuses; bus++)
                {
                int r = MustuxAudioDeviceMapper::open_device(bus,MustuxAudioDeviceMapper::PLAY,rate,channels,bitDepth);
                if (r<0)
                        {
                        QString sb; sb.setNum(bus);
                        song->info("BUS " + sb + " SEEMS UNAVAILABLE! Maybe busy ?");
                        PWARN("Bus %d seems unavailable.",bus);
                        busStatus[bus]=false;
                        }
                else
                        busStatus[bus]=true;
                }

        // JUST FOR DEBUGGING
        //show_slices();

        build_slice_list(lastMtaBlock);

        Slice* sl = firstSlice;
        currentSlice = sl;
        while ((playStartBlock < sl->leftBorder) ||  (playStartBlock > sl->rightBorder))
                {
                sl=sl->next;
                if (!sl)
                        break;
                else
                        currentSlice = sl;
                }

        blockOffset = playStartBlock - currentSlice->leftBorder;
        isPlaying=true;

        // just test
        //while (is_playing() && next_slice());
        //stop();

        busMonitor->activate();
        thPlayLoop->start();

        PEXIT;
        return 1;
        }


int Mixer::force_stop()
        {
        isPlaying=false;
        thPlayLoop->wait();
        return 1;
        }

int Mixer::stop()
        {
        PENTER;
        isPlaying=false;        
        int nBuses = MustuxAudioDeviceMapper::get_number_of_buses();
        for (int bus=0; bus<nBuses; bus++)
                {
                if (busStatus[bus]==false)
                        {
                        PMESG("Device for bus %d is unavailable, so I won't try to close it", bus );
                        }
                else
                        {
                        if (MustuxAudioDeviceMapper::close_device(bus)<0)
                                {
                                PERROR("Could not close device for bus %d",bus);
                                }
                        }
                }
        delete busStatus;
        bufPointer=0;
        busMonitor->deactivate();
        for (int i=0; i<song->get_total_tracks(); i++)
                song->track[i]->stop(); // cleanup_playback would be a good name instead of stop, in this case (tracks)

        PEXIT;
        return 1;
        }


int Mixer::build_slice_list(int lastBlockPos)
        {
        PENTER;
        PMESG("Starting building Slice List : Width = %d ",sliceWidth);
        firstSlice = (Slice*) 0;
        int ID=0;
        long long border = 0;
        bool first = true;
        Slice* s=0;
        while (lastBlockPos > border)
                {
                if (first)
                        {
                        s = new Slice(this, ID);
                        firstSlice=s;
                        firstSlice->prev=0;
                        first=false;
                        }
                else
                        {
                        s->next = new Slice(this, ID);
                        s->next->prev=s;
                        s=s->next;
                        }
                s->leftBorder=border;
                s->rightBorder=border+sliceWidth;
                //PMESG("Created Slice #%d LB=%d RB=%d",ID,s->leftBorder,s->rightBorder);
                ID++;
                border+=sliceWidth;
                }
        s->next=0;

        PMESG("Total slices created : %d",ID);

        // Finding sub-clips inside Slices...
        Slice* slice = firstSlice;
        int nsc = 0;
        while (slice)
                {
                long long sliceLBorder = slice->leftBorder;
                long long sliceRBorder = slice->rightBorder;
                int sliceWidth = sliceRBorder - sliceLBorder;
                int rsp;
                for (int i=0; i<song->get_total_tracks(); i++)
                        {
                        AudioClip* clip = song->track[i]->get_clip_under(sliceLBorder);
                        if (clip)
                                {
                                SubClip* sc;
                                long long sfb = clip->sourceFirstBlock + (sliceLBorder-clip->trackFirstBlock);
                                rsp=0;
                                if (clip->trackLastBlock>sliceRBorder)
                                        {
                                        int subclipWidth=sliceWidth;
                                        //PDEBUG("Found   full  Subclip in Slice #" << slice->ID << " under track " << i << " RSP=" << rsp << " , SFB=" << sfb << " , Width=" << subclipWidth);
                                        sc = new SubClip(clip, rsp, sfb, subclipWidth);
                                        }
                                else
                                        {
                                        int subclipWidth = clip->trackLastBlock - sliceLBorder;
                                        //PDEBUG("Found partial Subclip in Slice #" << slice->ID << " under track " << i << " RSP=" << rsp << " , SFB=" << sfb << " , Width=" << subclipWidth);
                                        sc = new SubClip(clip, rsp, sfb, subclipWidth);
                                        }
                                slice->add_sub_clip(sc,i);
                                nsc++;
                                }
                        clip = song->track[i]->get_next_clip_after(sliceLBorder);
                        while (clip && (clip->trackFirstBlock < sliceRBorder))
                                {
                                int rsp = clip->trackFirstBlock - sliceLBorder;
                                int sfb = clip->sourceFirstBlock;
                                if (clip->trackLastBlock < sliceRBorder)
                                        {
                                        int subclipWidth = clip->get_lenght();
                                        //PDEBUG("Found partial Subclip in Slice #" << slice->ID << " under track " << i << " RSP=" << rsp << " , SFB=" << sfb << " , Width=" << subclipWidth);
                                        SubClip* sc = new SubClip(clip, rsp, sfb, subclipWidth);
                                        slice->add_sub_clip(sc,i);
                                        nsc++;
                                        clip = song->track[i]->get_next_clip_after(clip->trackLastBlock);
                                        }
                                else
                                        {
                                        int subclipWidth = sliceRBorder-clip->trackFirstBlock;
                                        //PDEBUG("Found partial Subclip in Slice #" << slice->ID << " under track " << i << " RSP=" << rsp << " , SFB=" << sfb << " , Width=" << subclipWidth);
                                        SubClip* sc = new SubClip(clip, rsp, sfb, subclipWidth);
                                        slice->add_sub_clip(sc,i);
                                        nsc++;
                                        break;
                                        }
                                }
                        }
                slice=slice->next;
                }
        PMESG("Total subclips created : %d" ,nsc);
        //show_slice_list();
        PEXIT;
        return ( nsc > 0 ? 1 : -1 );
        }


void Mixer::show_slice_list()
        {
        PENTER;
        PDEBUG("Slice List :");
        Slice* s = firstSlice;
        while (s)
                {
                s->show();
                s=s->next;
                }
        PEXIT;
        }


long long Mixer::get_current_playing_block()
        {
        if (currentSlice)
                return currentSlice->leftBorder + relativeBlockInsideSliceBeingPlayed;
        else
                return 0;
        }



int Mixer::render_all(QString filename)
        {
        PENTER;

        PDEBUG("Rendering song into %s", (const char*)filename.latin1());
        
        // IMPROVEME. Get this from a dialog, make conversions, and so on...
        int rate=44100;
        int channels=2;
        int bitDepth=16;

        if (fragSize<0) fragSize=4096;
        if (!firstSlice)
                build_slice_list(song->get_last_song_block());

        bool renderToPraf=true;
        QString s(filename);
        if (s.find(".wav",0,false)>0)
                {
                renderToPraf=false;
                }

        Slice* sl = firstSlice;

        // count number of slices
        int ns = 0; while (sl) { ns++; sl=sl->next; }

        if (renderToPraf)
                {
                PrafFile* pf = new PrafFile();
                pf->create((char*)filename.latin1());
                pf->write_header(channels, rate, bitDepth, 0,
                                "RenderAllTest", "Test", "Test",
                                "Output audio from RenderAll action");

                sl = firstSlice;
                int i=0;
                while (sl)
                        {
                        //PMESG("Rendering slice %d" << i<< "/"<<ns << " to " << filename);
                        int maxRenderedByteInThisSlice = sl->render(renderBuffer);
                        if (sl->next)
                                {
                                PMESG("Appending %d bytes from slice %d ", renderBufferSize, sl->ID);
                                pf->append_samples_from_buffer( renderBuffer, renderBufferSize);
                                }
                        else
                                {
                                PMESG("Closing render with  %d bytes from slice %d ", maxRenderedByteInThisSlice, sl->ID);
                                pf->append_samples_from_buffer( renderBuffer, maxRenderedByteInThisSlice);
                                }
                        sl=sl->next;
                        i++;
                        }
                PMESG("Rendering Finished");
                pf->fix_audio_size();
                pf->close_file();
                delete pf;
                }
        else
                {
                WavFile* wf = new WavFile();
                wf->create((char*)filename.latin1());
                wf->write_header(channels, rate, bitDepth);

                sl = firstSlice;
                int i=0;
                while (sl)
                        {
                        //PMESG("Rendering slice " << i<< "/"<<ns << " to " << filename);
                        sl->render(renderBuffer);
                        wf->append_samples_from_buffer( renderBuffer, renderBufferSize);
                        sl=sl->next;
                        i++;
                        }
                PMESG("Rendering Finished");
                wf->fix_audio_size();
                wf->close_file();
                delete wf;
                }
        PEXIT;
        return 1;
        }


char* Mixer::get_current_mixed_audio_frag()
        {
        return bufPointer;
        }

int Mixer::get_current_mixed_audio_frag_size()
        {
        return fragSize;
        }

bool Mixer::is_playing()
        {
        return isPlaying;
        }

/*void Mixer::show_slices()
        {
        Slice* sl = firstSlice; while (sl) { sl->show(); sl=sl->next; }
        }
*/



bool Mixer::next_slice()
        {
        //PENTER;
        if (!currentSlice)
                return false;

        relativeBlockInsideSliceBeingPlayed = blockOffset;

        PMESG("Playing SLICE %d",currentSlice->ID);
        int nBuses = MustuxAudioDeviceMapper::get_number_of_buses();
        for (int bus=0; bus<nBuses; bus++)
                {
                // ignore those buses which are unavailable
                if (!busStatus[bus])
                        continue;

                // check if there is some track routing to this bus
                bool someTrackRoutesToThisBus=false;
                for (int i=0; i<song->get_total_tracks(); i++)
                        {
                        if (song->track[i]->get_bus_out()==bus)
                                {
                                someTrackRoutesToThisBus=true;
                                break;
                                }
                        }
                // if not, forget about this track
                if (!someTrackRoutesToThisBus)
                        continue;

                // if yes, mix current slice using only tracks which route to this bus
                int r = currentSlice->mix(bus, mixBuffer[bus]);

                bufPointer = mixBuffer[bus] + blockOffset * maxChannels * sizeof(short);
                endbufPointer = bufPointer + mixBufferSize[bus];
                int numBlocksInFrag = fragSize / ( maxChannels * sizeof(short));

                while (isPlaying && (bufPointer < endbufPointer))
                        {
                        busMonitor->update_levels(bus,bufPointer,fragSize);
                        MustuxAudioDeviceMapper::play_frag(bus,bufPointer);
                        bufPointer+= fragSize;
                        relativeBlockInsideSliceBeingPlayed=0;// += numBlocksInFrag; FIX ME
                        }

                }

        currentSlice = currentSlice->next;
        blockOffset = 0;

        //PEXIT;
        return true;
        }


ThPlayLoop::ThPlayLoop(Mixer* pMixer)
        {
        mixer=pMixer;
        }


void ThPlayLoop::run()
        {
        msleep(100);
        while (mixer->is_playing() && mixer->next_slice());
        mixer->stop();
        }


int Mixer::get_slice_width_in_bytes()
        {
        return sliceWidthInBytes;
        }

int Mixer::get_slice_width_factor()
        {
        return sliceWidthFactor;
        }

int Mixer::get_slice_width()
        {
        return sliceWidth;
        }

int Mixer::get_max_channels()
        {
        return maxChannels;
        }

Song* Mixer::get_parent_song()
        {
        return song;
        }


//eof

