/**************************************************************************************

        PROTUX - THE FREE PROFESSIONAL AUDIO TOOLS FOR LINUX
        AUTHOR : See AUTHORS file for details

        This software is distributed under the terms of the GNU General Public License
        as specified in the COPYING file.

***************************************************************************************/
#ifndef PRAFFILE_HH
#define PRAFFILE_HH

#include <stdio.h>

class WavFile;

class PrafFile
    {
    public:
        static const int PRAF_HEADER_LABEL_SIZE = 16;
        static const int PRAF_HEADER_DESCRIPTION_SIZE = 56;
        static const int PRAF_HEADER_SIZE = 256;
        static const int PRAF_HEADER_RESERVED_SIZE = 128;

        PrafFile();
        ~PrafFile();
        int open(char* pFileName);
        int create(char* pFileName);
        int copy_to(char* pFileName);
        int read_header();
        int write_header(int pChannels, int pRate, int pBitDepth, long long pAudioSize,
                         const char*  pProjectLabel, const char*  pGroupLabel, const char*  pHardwareLabel,
                         const char*  pDescription);
        int append_samples_from_wav(WavFile* wavfile);
        int append_samples_from_buffer(char* buffer, int bufferSize);
        void show_info();
        void rewind();
        void skip_header();
        char*  get_project_label();
        char*  get_group_label();
        char*  get_hardware_label();
        char*  get_description();
        int get_channels();
        int get_rate();
        int get_bit_depth();
        long long get_audio_size();
        int get_header_size();
        FILE* get_file();
        char*  get_filename();
        long long get_file_size();
        int fix_audio_size();

        int close_file();


    
    private:

        FILE* file;
        char*  fileName;
        char formatLabel[4];
        int channels;
        int rate;
        int bitDepth;
        long long audioSize;
        int headerSize;
        char projectLabel[PRAF_HEADER_LABEL_SIZE];
        char groupLabel[PRAF_HEADER_LABEL_SIZE];
        char hardwareLabel[PRAF_HEADER_LABEL_SIZE];
        char description[PRAF_HEADER_DESCRIPTION_SIZE];
        char reserved[PRAF_HEADER_RESERVED_SIZE];
        };

#include "WavFile.hh"


#endif

