/**************************************************************************************

        PROTUX - THE FREE PROFESSIONAL AUDIO TOOLS FOR LINUX
        AUTHOR : See AUTHORS file for details

        This software is distributed under the terms of the GNU General Public License
        as specified in the COPYING file.

***************************************************************************************/
#include "Recorder.hh"
#include "TrackPanelVU.hh"


// UNTIL TTD MAPPING IS DONE, THE RECORDER ENGINE WILL BE LINKED TO EACH
// TRACK. THIS IS WRONG, BECAUSE TOO DSP CONTROLLERS ARE CREATED
// THE CORRECT IS THE RECORDER TO BE A SONG LINKED ENGINE, AND HAVE
// A TTD MAPPING, BASE ON THE DEVICES AVAIABLE FROM MUSTUXDSP


Recorder::Recorder(Track* pTrack )
        {
        PENTERCONS;
        outputTrack = pTrack;
        targetSong=outputTrack->parentSong;

        channels=2;
        rate=44100;
        bitDepth=16;
        initialInsertBlockPos = (long long) 0;
        insertingBlockPos = (long long) 0;
        blockSize = channels * (bitDepth/8);
        blinkTimer = new QTimer(this);
        connect( blinkTimer , SIGNAL(timeout()), this, SLOT(blink_rec_status()));
        blinkStatus=false;
        recStatus=REC_NONE;
        inputLevel=new float[channels];
        max = new short[channels];
        thRecLoop = new ThRecLoop(this);

        busIn = MustuxAudioDeviceMapper::DEFAULT_BUS;

        PEXITCONS;
        }


Recorder::~Recorder()
        {
        PENTERDES;
        delete inputLevel;
        delete max;
        PEXITDES;
        }

void Recorder::set_output_file(QString fn)
        {
        outputFilename=fn;
        }



void Recorder::set_capture_dir(QString cd)
        {
        captureDir=cd;
        }


int Recorder::arm()
        {
        PENTER;
        int r = MustuxAudioDeviceMapper::open_device(busIn,MustuxAudioDeviceMapper::REC,rate,channels,bitDepth);
        if ( r==-1)
                {
                PERROR("Cannot Open audio device for Record");
                PEXIT;
                return -1;
                }
        if ( r==-2) // should be r==MustuxAudioDeviceMapper::ERROR_BUSY_BUS to be more clear...
                {
                QString mesg = "IN-BUS ";
                QString sbus; sbus.setNum(busIn);
                mesg.append(sbus);
                mesg.append(" IS BUSY");
                targetSong->info(mesg,1);
                PEXIT;
                return -1;
                }

        recBufferSize  =  MustuxAudioDeviceMapper::get_fragment_size(busIn);
        PMESG("Detected fragment size for REC : recBufferSize=%d",recBufferSize  );
        recBuffer = new char[recBufferSize];
        insertingBlockPos = targetSong->get_working_block();
        initialInsertBlockPos = insertingBlockPos;
        output = new Audio(targetSong);
        output->create(captureDir, outputFilename, channels, rate, bitDepth,
                       "Untitled", "UndefGroup", "UndefSoundCard","Real Time Take");
        outputFile = output->get_file();

        output->rewind();

        recStatus = REC_ARMED;
        thRecLoop->start();

        outputTrack->assocTrackPanel->get_vu()->activate();
        blinkTimer->start(75);


        PEXIT;
        return 1;
        }



int Recorder::rec()
        {
        PENTER;
        recStatus = REC_RECORDING;
        PEXIT;
        return 1;
        }


int Recorder::force_stop()
        {
        PENTER;
        recStatus = REC_FINISHED;
        thRecLoop->wait();
        PEXIT;
        return 1;
        }


int Recorder::stop()
        {
        PENTER;
        blinkTimer->stop();
        outputTrack->assocTrackPanel->get_vu()->deactivate();
        int r= MustuxAudioDeviceMapper::close_device(busIn);
        if (r<0)
                {
                PERROR("Cannot close audio device. Continuing anyway ...");
                }
        if (recStatus==REC_FINISHED)
                {
                output->fix_size();
                output->rebuild_peaks();
                AudioClip* clip = targetSong->create_clip(output, true, outputTrack, initialInsertBlockPos);
                if (!clip)
                        {
                        PERROR("After recording, could not create clip with recorded audio! Continuing anyway ...");
                        }
                outputTrack->recreate();
                }
        else if (recStatus==REC_ABORTED)
                {
                delete output;
                }
        delete recBuffer;
        recStatus=REC_ARMED;
        PEXIT;
        return 1;
        }


int Recorder::abort()
        {
        recStatus = REC_ABORTED;
        thRecLoop->wait();
        PEXIT;
        return 1;
        }

int Recorder::disarm()
        {
        abort();
        recStatus = REC_NONE;
        PEXIT;
        return 1;
        }


void Recorder::blink_rec_status()
        {
        QString bs;
        if (recStatus==REC_ARMED)
                {
                bs="ARMED";
                }
        else if (recStatus==REC_RECORDING)
                {
                bs="RECORDING...";
                }
        if (blinkStatus)
                {
                targetSong->info(bs);
                }
        else
                {
                targetSong->info("          ");
                }
        blinkStatus=!blinkStatus;
        }




int Recorder::get_status()
        {
        return recStatus;
        }

void Recorder::monitor_input()
        {
        for (int k=0; k<channels; k++) max[k] = 0;
        short sho;
        char* p = recBuffer;
        while ( p < recBuffer + recBufferSize )
                {
                for (int k=0; k<channels; k++)
                        {
                        sho = (int)*(short*)p;
                        if ( sho > max[k] ) max[k]=sho;
                        p += sizeof(short);
                        }
                }
        for (int k=0; k<channels; k++)
                inputLevel[k] = (float) max[k] / 32767.0;
        outputTrack->assocTrackPanel->vu->update_levels(inputLevel);
        }


void Recorder::draw_input()
        {
        int rx = targetSong->block_to_xpos(insertingBlockPos);
        int totalSamples = recBufferSize/blockSize;
        int numSamplesToDraw = targetSong->block_to_xpos(totalSamples);
        MustuxDrawable* drawArea = outputTrack->get_drawarea();
        if ( rx > drawArea->width() )
                targetSong->scroll_right(drawArea->width()); // NOT WORKING
        else if ( rx >= 0 )
                {
                if ( rx + numSamplesToDraw > drawArea->width() )
                        numSamplesToDraw = drawArea->width() - rx;
                int recBufferStep = targetSong->get_zoom_step()*blockSize;
                int sho,maxsho;
                int height = outputTrack->height;
                int chhalf = (height-12) / channels;
                int x = rx;
                int sampleCounter=0;
                if (numSamplesToDraw==0)
                        {
                        recBufferStep = recBufferSize;
                        numSamplesToDraw=1;
                        }
                char* posBuffer = recBuffer;
                int baseY = outputTrack->baseY;
                while ( sampleCounter < numSamplesToDraw )
                        {
                        for (int k=0; k < channels; k++)
                                {
                                int yc = baseY + chhalf*(k+1) + 4 ;
                                drawArea->painter->setPen(QColor(150,120,120));
                                drawArea->painter->drawLine(x,yc,x,yc-chhalf+1);
                                char* p = posBuffer + k*sizeof(short);
                                maxsho=0;
                                while ( p < posBuffer + recBufferStep )
                                        {
                                        sho = (int)*(short*)p;
                                        if ( sho > maxsho ) maxsho=sho;
                                        p += sizeof(short)*channels;
                                        }
                                if ( maxsho >= 32767 )
                                        {
                                        maxsho = 32767;
                                        // printf(" *CLIP* ");
                                        drawArea->painter->setPen(Qt::red);
                                        inputLevel[k]=maxsho;
                                        }
                                else
                                        drawArea->painter->setPen(Qt::yellow);
                                float f = (float) maxsho / 32768;
                                int y = (int) (f*chhalf);
                                drawArea->painter->drawLine(x,yc,x,yc-y);
                                }
                        posBuffer+=recBufferStep;
                        x++;
                        sampleCounter++;
                        }
                drawArea->update(rx,baseY,numSamplesToDraw,height);
                }
        }


bool Recorder::rec_frag()
        {
        if ( MustuxAudioDeviceMapper::rec_frag(busIn,recBuffer)<0)
                {
                PERROR("Cannot read from bus %d",busIn);
                recStatus=REC_ABORTED;
                return false;
                }

        monitor_input();

        // write to disk in case of recording
        if (recStatus==REC_RECORDING)
                {
                if ( fwrite( recBuffer, recBufferSize, 1, outputFile ) == 0)
                        {
                        PERROR("Cannot write to output file.");
                        recStatus=REC_ABORTED;
                        return false;
                        }
                draw_input();
                insertingBlockPos += recBufferSize/blockSize;
                }
        return true;
        }




ThRecLoop::ThRecLoop(Recorder* pRecorder)
        {
        recorder = pRecorder;
        }


void Recorder::set_bus_in(int bus)
        {
        busIn = bus; // need some kind of check...
        }

int Recorder::get_bus_in()
        {
        return busIn;
        }

void ThRecLoop::run()
        {
        msleep(100);
        PENTER; 
        while (true)
                {
                if ((recorder->get_status()==Recorder::REC_FINISHED) || (recorder->get_status()==Recorder::REC_ABORTED) || (!recorder->rec_frag()))
                         break;
                }
        recorder->stop();
        PEXIT;
        }

//eof

