/**************************************************************************************

        PROTUX - THE FREE PROFESSIONAL AUDIO TOOLS FOR LINUX
        AUTHOR : See AUTHORS file for details

        This software is distributed under the terms of the GNU General Public License
        as specified in the COPYING file.

***************************************************************************************/
#ifndef SONG_HH
#define SONG_HH

#include <qwidget.h>
#include <qlayout.h>

#include <mustux.h>

class MTALocator;
class AudioSourcesList;
class Project;
class Track;
class TrackPanel;
class Audio;
class Mixer;
class Track;
class AudioClip;
class Interface;
class Spectrogram;
class CursorManager;
class Locator;
class TrackPanelsArea;
class TracksArea;
class Slice;
class FilterSelector;
class FilterController;
class BusSelector;
class BusMonitor;


class Song : public QWidget
        {
        Q_OBJECT

        public:
                static const int MAX_TRACKS = 32;
                static const int JOG_NONE = 0;
                static const int JOG_DRAG = 1;
                static const int JOG_ZOOM = 2;
                static const int JOG_SHUTTLE = 3;
                static const int JOG_EDGE = 4;
                static const int JOG_GAIN = 5;
                static const int JOG_PAN = 6;
                static const int JOG_GAIN_AND_PAN = 9;
                static const int LEFT_EDGE = 1;
                static const int RIGHT_EDGE = 2;
                static const int BOTH_EDGES = 3;

                static const int STOPPED = 0;
                static const int ARMED = 1;
                static const int RECORDING = 2;
                static const int PLAYING = 3;
                static const int RENDERING = 4;

                static const int EDITING_MODE_NORMAL = 1;
                static const int EDITING_MODE_TRACK_CURVES = 2;


                Song(int pMyNumber, Project* pParentProject, QWidget* parent);
                ~Song();

                // general methods
                Project* get_parent_project();
                AudioSourcesList* get_asl();
                MustuxDrawable* get_tracks_area();
                MustuxDrawable* get_track_panels_area();
                MustuxLcd* get_default_lcd();

                int process_jmb_action(MustuxJogMouseBoardMessage* m);
                int load();
                int save();
                int render();
                int render(QString filename);
                int toggle_play();
                int toggle_record();

                int get_status();

                // track methods
                void         set_active_track(int trackNumber);
                int          get_active_track_number();
                Track*       get_active_track();
                Track*       get_track(int trackNumber);
                TrackPanel*  get_active_track_panel();
                Track*       get_track_under_mouse();
                Track*       get_track_under_y (int y);
                TrackPanel*  get_track_panel_under_y (int y);
                int          add_track();
                int          add_track(int trackNumber);

                // Track Curve methods
                //int add_track_env_node(QString whichEnvCurve, int x, int y);

                // Song atributes methods
                void set_artists(QString pArtistis);
                void set_master_gain(float pMasterGain);
                QString get_artists();
                float get_master_gain();

                // Audio Methods
                int import_audio(QString filename);
                void add_audio_source(Audio* pAudio);
                void delete_audio_source(Audio* pAudio);

                // Audio clips
                AudioClip* create_clip(QString audiofileRootdir, QString filename);
                AudioClip* create_clip(Audio* au, bool isTake=false);
                AudioClip* create_clip(Audio* au, bool isTake, Track* t, long long insertPos);
                AudioClip* get_clip_under_XY(int x, int y);
                int get_mouse_x();
                int get_mouse_y();

                void disable_cursors();
                void enable_cursors();

                // Actions
                void recreate();
                void touch();
                void touch_x(int x);
                void touch_track(int trackNumber);
                void touch_in_active_track();
                void touch_in_active_track(int x);
                void touch_in_active_track_panel();
                void touch_in_active_track_panel(int x);

                void update_pan(int x);
                void update_gain(int y);
                void jog_gain(int track);
                void jog_pan(int track);
                void jog_gain_pan(int track);

                int split();
                int mute();
                int solo();
                int select();
                int process_delete();
                int drag(bool pIsCopying);
                int drag_edge(int mode);
                void jog_zoom();
                void update_jog_zoom(int x);
                void update_shuttle_factor(int x);
                void update_drag_edge(int x, int y);
                void set_hzoom_level(int level);
                void hzoom_in();
                void hzoom_out();
                void vzoom_in();
                void vzoom_out();
                int get_hzoom();
                int get_vzoom();
                int get_zoom_step();
                void goto_begin();
                void goto_end();
                void center();
                void shuttle();
                void scroll_left(int amount);
                void scroll_right(int amount);
                void set_first_block(long long pos);
                


                // other methods
                int get_total_tracks();
                QString get_capture_dir();
                QString get_mta_schema();
                void set_output_spectrogram(Spectrogram* spectrogram);
                void info(QString message);
                void info(QString message, int line);                
                void show_zoom_info();

                int get_editing_mode();

                int block_to_xpos(long long block);
                long long xpos_to_block(int xpos);
                long long get_first_block();
                long long get_last_block();
                long long get_working_block();
                long long workingBlock;
                long long firstBlock;
                long long lastBlock;
                long long get_playing_block();
                int get_playing_xpos();
                long long get_last_song_block();

                BusMonitor* get_bus_monitor();

        protected:
                Project* parentProject;
                TrackPanelsArea* trackPanelsArea;
                TracksArea* tracksArea;
                Locator* locator;
                AudioSourcesList* audioSourcesList;
                CursorManager* cursorManager;
                Mixer* mixer;
                Track* track[MAX_TRACKS];
                TrackPanel* trackPanel[MAX_TRACKS];
                AudioClip* workingClip;
                QTimer* shuttleTimer;

                QString artists;
                QString title;
                float masterGain;
                int myNumber;
                int numTracks;
                int activeTrackNumber;
                int trackHeight;
                QString rootDir;
                QString audioSourcesDir;
                QString captureDir;
                QString mtaProj;


                int jogMode;
                int takeNumber;
                int hzoom;
                int vzoom;
                int draggingEdge;
                int jogZoomTotalX;
                int shuttleFactor;
                int lastJogZoomFactor;
                int baseJogZoomFactor;
                bool isScrolling;
                bool isCopying;
                int gainingTrack;
                int origX;
                int origY;

                float origGain;

                int editingMode;

                void resizeEvent(QResizeEvent* e);
                void mouseMoveEvent(QMouseEvent* e);
                void recreate_tracks();
                void recreate_panels();

                FilterSelector* filterSelector;
                BusSelector* busSelector;


                friend class Mixer;
                friend class Slice;
                friend class TrackPanel;
                friend class Track;
                friend class AudioClip;
                friend class CursorManager;
                friend class Locator;
                friend class TracksArea;
                friend class TrackPanelsArea;
                friend class FilterController;

        public slots :
                void update_shuttle();
                void cleanupEventFilter(); // Just to avoid the warning. See the comment in the implementation


        };

#include "Interface.hh"
#include "Project.hh"
#include "Track.hh"
#include "TrackPanel.hh"
#include "Mixer.hh"
#include "Audio.hh"
#include "AudioClip.hh"
#include "AudioSourcesList.hh"
#include "Spectrogram.hh"
#include "CursorManager.hh"
#include "Locator.hh"
#include "TracksArea.hh"
#include "TrackPanelsArea.hh"
#include "filters/Filters.hh"
#include "BusSelector.hh"
#include "BusMonitor.hh"

#endif



//eof
