/**************************************************************************************

        PROTUX - THE FREE PROFESSIONAL AUDIO TOOLS FOR LINUX
        AUTHOR : See AUTHORS file for details

        This software is distributed under the terms of the GNU General Public License
        as specified in the COPYING file.

***************************************************************************************/
#ifndef TRACK_HH
#define TRACK_HH

#include <mustux.h>


class Audio;
class AudioClip;
class Mixer;
class Song;
class TrackPanel;
class FilterChain;
class Recorder;

class Track : public MustuxDrawableRegion
        {
        public :
                Track(Song* pParentSong, int pBaseY, int pHeight);
                ~Track();

                void set_assoc_track_panel(TrackPanel* pAssocTrackPanel);
                void recreate();
                void activate();
                void deactivate();
                void update();
                int play(long long startpoint);
                int stop();
                int draw();
                void clear();
                void clear_clips();
                bool is_pointed();
                int get_recorder_status();
                void set_blur(bool stat);

                AudioClip* add_clip(Audio* audioSource, long long insertBlock, bool isTake = false);
                AudioClip* add_clip(Audio* audioSource, long long insertBlock, long long sourceFirstblock, long long lenght, bool isTake = false);
                AudioClip* get_clip_under(long long blockPos);
                AudioClip* get_next_clip_after(long long blockPos);

                int split_clip(long long splitPoint);
                int delete_clip(AudioClip* clip);

                AudioClip* clipList;

                int get_base_Y() { return baseY; };
                void toggle_active_rect();

                int toggle_recordable();
                int toggle_record();
                void draw_recording_peaks(char* recBuf, int bufSize, int blockSize, int channels, long long startPos);

                QString get_name();

                Song* get_parent_song();

                int get_bus_in();
                int get_bus_out();

                void set_bus_in(int b);
                void set_bus_out(int b);

                bool is_muted();
                bool is_solo();
                bool is_recordable();
                bool is_active();

                void toggle_active();
                
                QString get_schema();

        private :
                TrackPanel* assocTrackPanel;
                Song* parentSong;
                FilterChain* filterChain;
                Recorder* recorder;
                int baseY;
                int height;
                int busIn;
                int busOut;

                bool isActive;
                bool isSolo;
                bool isMuted;

                int get_baseX();
                int get_baseY();
                int get_width();
                int get_height();


                friend class Song;
                friend class AudioClip;
                friend class Mixer;
                friend class Slice;
                friend class Curve;
                friend class TrackPanel;
                friend class Recorder;
                friend class FilterChain;


        };

#include "Song.hh"
#include "Slice.hh"
#include "Audio.hh"
#include "AudioClip.hh"
#include "Mixer.hh"
#include "TrackPanel.hh"
#include "filters/Filters.hh"
#include "Recorder.hh"

#endif



