/**************************************************************************************

        PROTUX - THE FREE PROFESSIONAL AUDIO TOOLS FOR LINUX
        AUTHOR : See AUTHORS file for details

        This software is distributed under the terms of the GNU General Public License
        as specified in the COPYING file.

***************************************************************************************/
#include "CurveNode.hh"

CurveNode::CurveNode(Curve* pParentCurve, long long pPos, float  pValue)
        {
        PENTERCONS;
        parentCurve = pParentCurve;
        set_pos(pPos);
        set_value(pValue);
        isHighLighted=false;
        PEXITCONS;
        }

CurveNode::~CurveNode()
        {
        PENTERDES;
        PEXITDES;
        }

void CurveNode::toggle_highlight()
        {
        isHighLighted=!isHighLighted;
        draw();
        }

void CurveNode::set_pos(long long pPos)
        {
        if (pPos<0) pPos = 0;
        pos = pPos;
        }


void CurveNode::set_value(float pValue)
        {
        // Standard range : -100 to + 100 dB. Used as % when drawing
        if (pValue>100.0) pValue=100.0;
        if (pValue<-100.0) pValue=-100.0;
        value=pValue;
        }



void CurveNode::draw()
        {
        FilterController* fc = parentCurve->assocFilterController;
        MustuxDrawable* drawArea = fc->get_drawarea();
        QPainter* painter = drawArea->painter;

        const int NODE_WIDTH=10;

        if (this->isHighLighted)
                painter->setPen(QColor(0, 255, 0));

        else
                painter->setPen(QColor(0,0,0));

        int baseY = fc->get_baseY();
        int half = fc->get_height()/2;
        int yc = baseY + half;
        int x = parentCurve->parentSong->block_to_xpos(pos);
        int y = yc + (int) (-1*value*half/100);


        // MODE SUPERMARKERS
        /*for (int k=-NODE_WIDTH; k<NODE_WIDTH; k++)
                        {
                        painter->drawLine(x+k, baseY, x, y);
                        painter->drawLine(x+k, floorY, x, y);
                        }
        */

        // MODE CLASSICAL NODE
        int x1=x-(NODE_WIDTH/2);
        int y1=y-(NODE_WIDTH/2);
        painter->drawRect(x1,y1,NODE_WIDTH,NODE_WIDTH);
        drawArea->update(x1,y1,NODE_WIDTH,NODE_WIDTH);
        }

void CurveNode::setup()
        {
        parentCurve->assocFilterController->setupFrontend->show();
        }


//eof
