/**************************************************************************************

        PROTUX - THE FREE PROFESSIONAL AUDIO TOOLS FOR LINUX
        AUTHOR : See AUTHORS file for details

        This software is distributed under the terms of the GNU General Public License
        as specified in the COPYING file.

***************************************************************************************/

#include "FilterChain.hh"
#include "Filters.hh"

FilterChain::FilterChain(Track* pAssocTrack)
        {
        PENTERCONS;
        for (int k=0; k<MAX_FILTER_CONTROLLERS; k++)
                filterController[k] = (FilterController*) 0;
        currentFilterController=-1;
        assocTrack=pAssocTrack;
        parentSong=assocTrack->parentSong;
        drawArea = assocTrack->drawArea;
        draggingNode = false;
        isSelectingController = false;
        lcd = parentSong->get_default_lcd();
        PEXITCONS;
        }


FilterChain::~FilterChain()
        {
        PENTERDES;
        for (int k=0; k<MAX_FILTER_CONTROLLERS; k++)
                if (filterController[k])
                        delete filterController[k];
        PEXITDES;
        }

void FilterChain::draw()
        {
        int baseY = assocTrack->get_baseY();
        int height = assocTrack->get_height();

        // draw the controllers queue
        drawArea->painter->setPen(QColor(0,0,100));
        drawArea->painter->setFont(QFont("Fixed",6));
        int by = baseY + height - 25;
        int y1 = by + 6;
        QColor cf(0,0,0);
        int j=0;
        for (int k=0; k<MAX_FILTER_CONTROLLERS; k++)
                {
                if (filterController[k])
                        {
                        int bx = 5 + j*80;
                        int x1 = bx + 70;
                        QString filterType = filterController[k]->get_type();
                        if (filterController[k]->get_filter()->isBypassed)
                                cf.setRgb(220,100,100);
                        else
                                {
                                if (k==currentFilterController)
                                        {
                                        filterController[k]->get_current_curve()->set_color(200,255,200);
                                        cf.setRgb(200,255,200);
                                        }
                                else
                                        {
                                        filterController[k]->get_current_curve()->set_color(200,200,250);
                                        cf.setRgb(200,200,250);
                                        }
                                }
                        QString currentCurveName = filterController[k]->get_current_curve()->get_type();
                        drawArea->painter->fillRect(bx,   by,    70, 19, cf);
                        drawArea->painter->drawRect(bx,   by,    70, 19);
                        drawArea->painter->drawText(bx+3, by+9, filterType);
                        drawArea->painter->drawText(bx+3, by+17, currentCurveName);
                        drawArea->painter->moveTo(x1,   y1);
                        drawArea->painter->lineTo(x1+5, y1);
                        drawArea->painter->lineTo(x1+5, y1-4);
                        drawArea->painter->lineTo(x1+8, y1+4);
                        drawArea->painter->lineTo(x1+5, y1+11);
                        drawArea->painter->lineTo(x1+5, y1+7);
                        drawArea->painter->lineTo(x1,   y1+7);
                        j++;
                        }
                }

        for (int k=0; k<MAX_FILTER_CONTROLLERS; k++)
                {
                if (filterController[k])
                        filterController[k]->draw();
                }
        }


FilterController* FilterChain::add_filter_controller(QString filterName)
        {
        PENTER;
        int fcIndex=-1;
        for (int k=0; k<MAX_FILTER_CONTROLLERS; k++)
                {
                if (!filterController[k])
                        {
                        fcIndex=k;
                        break;
                        }
                }
        if (fcIndex<0)
                {
                PERROR("Max number of filter controllers reached for track %s", (const char*) assocTrack->get_name().latin1());
                PEXIT;
                return (FilterController*) 0;
                }
        PMESG("Allocating %s controller in slot %d",(const char*) filterName.latin1() ,fcIndex);
        if (filterName=="")
                {
                parentSong->get_default_lcd()->print("Ok... no filter added :-/",1);
                PEXIT;
                return (FilterController*) 0;
                }
        if (filterName=="Gain")
                {
                Gain* filter = new Gain(44100,2);
                filterController[fcIndex]=new GainController(parentSong,drawArea,filter);
                }
        else
        if (filterName=="Pan")
                {
                Pan* filter = new Pan(44100,2);
                filterController[fcIndex]=new PanController(parentSong,drawArea,filter);
                }
        else
        if (filterName=="Reverb")
                {
                Reverb* filter = new Reverb(44100,2);
                filterController[fcIndex]=new ReverbController(parentSong,drawArea,filter);
                }
        else
        if (filterName=="Noise Gate")
                {
                NoiseGate* filter = new NoiseGate(44100,2);
                filterController[fcIndex]=new NoiseGateController(parentSong,drawArea,filter);
                }
        else
        if (filterName=="Octaver")
                {
                Octaver* filter = new Octaver(44100,2);
                filterController[fcIndex]=new OctaverController(parentSong,drawArea,filter);
                }
        else
        if (filterName=="Delay")
                {
                Delay* filter = new Delay(44100,2);
                filterController[fcIndex]=new DelayController(parentSong,drawArea,filter);
                }

        //else if (filterName=="...
        else
                {
                parentSong->get_default_lcd()->print("Filter/Controller not implemented yet",1);
                PEXIT;
                return (FilterController*) 0;
                }

        filterController[fcIndex]->set_baseX(0);
        filterController[fcIndex]->set_baseY(assocTrack->get_baseY());
        filterController[fcIndex]->set_height(assocTrack->get_height());
        filterController[fcIndex]->set_width(-1);
        // this should be inside FilterController constructor.... :-/
        // filterController[fcIndex]->setupFrontend->setCaption(filterController[fcIndex]->get_type()+" Controller Setup");
        set_current_filter_controller(fcIndex);
        assocTrack->recreate();
        PEXIT;
        return filterController[currentFilterController];
        }

void FilterChain::set_current_filter_controller(int index)
        {
        if (currentFilterController>=0)
                filterController[currentFilterController]->deactivate();
        currentFilterController = index;
        if (parentSong->get_editing_mode()==Song::EDITING_MODE_TRACK_CURVES)
                filterController[currentFilterController]->activate();
        }


void FilterChain::select_controller(bool isBeginning)
        {
        isSelectingController = isBeginning;
        }


int FilterChain::remove_controller()
        {
        PENTER;
        if (currentFilterController>=0)
                {
                delete (filterController[currentFilterController]);
                filterController[currentFilterController] = (FilterController*) 0;
                currentFilterController=-1;
                for (int k=0; k<MAX_FILTER_CONTROLLERS; k++)
                        {
                        if (filterController[k])
                                {
                                currentFilterController=k;
                                break;
                                }
                        }
                }
        assocTrack->recreate();
        PEXIT;
        return 1;
        }


void FilterChain::followMouse(int x, int y)
        {
        // CONTROLLER SELECTION
        if (isSelectingController)
                {
                int ncs=0;
                for (int k=0; k<MAX_FILTER_CONTROLLERS; k++)
                        if (filterController[k]) ncs++;
                if (ncs==0)
                        return;
                int index = x/(assocTrack->get_width()/ncs) + 1;
                //PMESG("Selecting Controller : " << index);
                int i=0;
                for (int k=0; k<MAX_FILTER_CONTROLLERS; k++)
                        {
                        if (filterController[k]) i++;
                        if (i==index)
                                {
                                set_current_filter_controller(k);
                                break;
                                }
                        }
                assocTrack->recreate();
                return;
                }

        // NODE/CURVE MANIPULATION
        if (
                (currentFilterController<0) ||
                (parentSong->get_editing_mode()!=Song::EDITING_MODE_TRACK_CURVES) ||
                ((!draggingNode) && (!assocTrack->is_pointed()))
           )
                return;


        long long lpos = parentSong->xpos_to_block(x);
        if (lpos<0) return;

        float cv = filterController[currentFilterController]->get_current_curve()->get_value_at(lpos);

        QPainter* p = lcd->painter;
        lcd->clear(550,100,100,20);
        p->setPen(Qt::black);
        QString scv; scv.setNum(cv);
        p->setFont(QFont("Fixed",8));
        p->drawText(560,100,scv);
        lcd->update(550,100,100,20);

        if (draggingNode)
                {
                CurveNode* cn = filterController[currentFilterController]->get_current_curve()->get_current_curve_node();
                if (cn->prev) // This is to avoid changing the root node position
                        {
                        long long pos = parentSong->xpos_to_block( x - originX ) + originPos;
                        cn->set_pos(pos);
                        }
                int half = assocTrack->get_height()/2;
                int dy = y - originY;
                float variation = -100.0 * ( ( (float) ( dy - half) / half ) + 1.0 );
                float v = originValue + variation;
                cn->set_value(v);
                assocTrack->recreate();
                }
        else if (assocTrack->is_pointed())
                filterController[currentFilterController]->update_current_node(x);

        }

void FilterChain::drag_node(bool isBeginning)
        {
        draggingNode = isBeginning;
        if ((draggingNode) && (currentFilterController>=0))
                {
                originX = drawArea->get_mouse_x();
                originY = drawArea->get_mouse_y();
                originPos   = filterController[currentFilterController]->get_current_curve()->get_current_curve_node()->pos;
                originValue = filterController[currentFilterController]->get_current_curve()->get_current_curve_node()->value;
                //PDEBUG("originX="<<originX<<"  originY="<<originY<< " \t originPos="<<originPos<< " originValue="<<originValue);
                }
        }


void FilterChain::add_node()
        {
        if (currentFilterController>=0)
                {
                int x = drawArea->get_mouse_x();
                int y = drawArea->get_mouse_y();
                long long pos  = parentSong->xpos_to_block(x);
                int half = assocTrack->get_height()/2;
                float value = -100.0 * ((float) ((y - assocTrack->get_baseY()) - half) / half );
                filterController[currentFilterController]->get_current_curve()->add_node(pos, value);
                assocTrack->recreate();
                }
        }


void FilterChain::node_setup()
        {
        if (currentFilterController>=0)
                {
                filterController[currentFilterController]->node_setup();
                assocTrack->recreate();
                }
        }


void FilterChain::filter_setup()
        {
        if (currentFilterController>=0)
                {
                filterController[currentFilterController]->setup();
                assocTrack->recreate();
                }
        }

int FilterChain::process(char* data, int size)
        {
        for (int k=0; k<MAX_FILTER_CONTROLLERS; k++)
                {
                if (filterController[k])
                        filterController[k]->process(data,size);
                }
        return 1;
        }

int FilterChain::cleanup()
        {
        for (int k=0; k<MAX_FILTER_CONTROLLERS; k++)
                {
                if (filterController[k])
                        filterController[k]->cleanup();
                }
        return 1;
        }


void FilterChain::deactivate()
        {
        if (currentFilterController>=0)
                filterController[currentFilterController]->deactivate();
        }


void FilterChain::activate()
        {
        if (currentFilterController>=0)
                filterController[currentFilterController]->activate();
        // maybe some things more here ...
        }


QString FilterChain::get_schema()
        {
        PENTER;
        QString schema="";
        for (int k=0; k<MAX_FILTER_CONTROLLERS; k++)
                {
                if (filterController[k])
                        {
                        schema.append("       <filtercontroller");
                        schema.append(" type=\""+filterController[k]->get_type()+"\"");
                        schema.append(">\n");
                        schema.append(filterController[k]->get_schema());
                        schema.append("       </filtercontroller>\n");
                        }
                }
        PEXIT;
        return schema;
        }

//eof
